package actinScope;
/*
 * Filament.java
 *
 */
/*  <ActinScope - for the simulation of dendritic actin networks>
    Copyright (C) <2005>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


import java.awt.*;
import javax.swing.*;
import java.lang.Math.*;
import com.sun.j3d.utils.geometry.Cylinder;
import com.sun.j3d.utils.geometry.Primitive;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.geometry.Box;
import com.sun.j3d.utils.universe.*;
//import com.sun.rsasign.g;

import javax.media.j3d.*;
import javax.vecmath.*;

public class Filament extends Thing {
	static Filament [] theFilaments = new Filament [100000];	// array holding all filaments
	static int filamentCt = 0;
	static double monosize=Env.gActinDiameter;
	static double halfmono=monosize/2;
	static double quartmono=monosize/4;
	static int nucCount = 3;		// 3 monomer nucleation size
	static double nuclength = halfmono*nucCount;
	static double radius = Env.fActinDiameter/2.0;		// (nm) radius of actin filament

	Pt3D end1 = new Pt3D();
	Pt3D end2 = new Pt3D();
	//Monomer [] mons = new Monomer [100];
	//int monCt = 0;
	Monomer plusMon = null;
	Monomer minusMon = null;
	double end1TipC = 1e6; // large number for initial tip clearance of end1
	double end2TipC = 1e6; // large number for initial tip clearance of end2
	boolean updateGraphicsLength = false;	// for graphics ONLY
	boolean updateGraphicsCapState = false;	// for graphics ONLY
	boolean end1Capped = false;
	boolean end2Capped = false;
	boolean isSideBranch = false;
	double length;
	
	//side-branches
	double end1CapOffset = 0.5*monosize;
	double end1BranchOffset = 1.8*monosize;
	Filament [] branches = new Filament [5];
	double [] branchLocs = new double [5];
	int branchCt = 0;
	
	
	int myFilamentNber;
	
	
	// for graphics
	BranchGroup cylBG;
	TransformGroup cylTG,end1TG,end2TG,capTG,arpTG;
	Transform3D cylT3D,end1T3D,end2T3D,capT3D,arpT3D;
	LineArray theFil;
	Shape3D filShape;
	Cylinder myCyl;
	Sphere end1S,end2S,arpS,capS;
	Appearance aSph,aCap,aArp,aCyl;
	Material capM,arpM,cylM;
	static Color3f ambientC = new Color3f(Color.BLACK);
	static Color3f diffuseC = new Color3f(Color.RED);
	static Color3f specularC = new Color3f(Color.WHITE);
	static Color3f emissiveC = new Color3f(Color.BLUE);
	static Color3f catasphColor = new Color3f(1.0f,1.0f,0.0f);
	static Color3f gotPlasmidColor = new Color3f(0.0f,1.0f,1.0f);
	static Color3f capBoxColor = new Color3f(1.0f,1.0f,1.0f);
	static Color3f arpSphColor = new Color3f(0.0f,1.0f,0.0f);
	static Color3f cylColor = new Color3f(1.0f,0.0f,0.0f);
	static float shiny = 128.0f;
	
	public Filament (Pt3D initCoord, Pt3D initAngle) {
		super(initCoord, initAngle);
		length=nuclength;
		theCell.takeActin((int)(nuclength/halfmono));
		addFilament(this);
		initialize();
	}

	public Filament (Pt3D initCoord, Pt3D initAngle, double length) {
		super(initCoord,initAngle);
		this.length = length ;
		theCell.takeActin((int)(length/halfmono));
		addFilament(this);
		initialize();
	}
	
	protected void sepaku() {
		super.sepaku();
		end1 = null;
		end2 = null;

		branches = null;
		
		// for graphics
		cylBG = null;
		cylTG = null;
		end1TG = null;
		end2TG = null;
		capTG = null;
		arpTG = null;
		cylT3D = null;
		end1T3D = null;
		end2T3D = null;
		capT3D = null;
		arpT3D = null;
		theFil = null;
		filShape = null;
		myCyl = null;
		end1S = null;
		end2S = null;
		arpS = null;
		capS = null;
		aSph = null;
		aCap = null;
		aArp = null;
		aCyl = null;
		capM = null;
		arpM = null;
		cylM = null;
		// clear the monomers
		Monomer curMon = minusMon;
		Monomer tempMon;
		while (curMon.plusMon != null) {
			tempMon = curMon.plusMon;
			curMon.sepaku();
			curMon = tempMon;
		}
	}
	
	public void initialize () {
		// calculate cos and sin of Euler angles only once during each time step
		cosAng.cosine(angle);
		sinAng.sine(angle);
		// find the transformation matrices at this time step
		transMat ();
		// find the unit vector that defines the orientation of the rod
		uVec.xToX(this, xUnitVector);
		t1Vec.xToX(this, yUnitVector);
		t2Vec.xToX(this, zUnitVector);
		// find the end points of the rod
		end1.add(coord, -length/2, uVec);
		end2.add(coord, length/2, uVec);
		makeInitMonomers();
	}
	
	public void slideFilament (double slideDist) {
		coord.inc(slideDist,uVec);
		end1.inc(slideDist,uVec);
		end2.inc(slideDist,uVec);
		// slide each Monomer as well, beginning with the minus end
		Monomer curMon = minusMon;	
		while (curMon.plusMon != null) {
			curMon.slideMonomer(slideDist,uVec);
			curMon = curMon.plusMon;	// next monomer
		}
	}
	
	public void addPlusMonomer () {
		Monomer newMon = addMonomer(end2);
		newMon.minusMon = plusMon;
		plusMon = newMon;
	}
	
	public void addMinusMonomer () {
		Monomer newMon = addMonomer(end1);
		newMon.plusMon = minusMon;
		minusMon = newMon;
	}
	
	public Monomer addMonomer (Pt3D pos) {
		Monomer newMon = new Monomer (pos,this);
		if (Env.eachMonomerASphere) { G.addChild(newMon.getGraphics());}
		return newMon;
	}
	
	public void removePlusMonomer () {
		Monomer rmMon = plusMon;
		if (plusMon.minusMon != null){
			plusMon = plusMon.minusMon;			//reset pointer to the plus-end monomer to the one directly behind the current plusMon
		}
		removeMonomer(rmMon);
	}
	
	public void removeMinusMonomer () {
		Monomer rmMon = minusMon;
		if (minusMon.plusMon != null){
			minusMon = minusMon.plusMon;
		}
		removeMonomer(rmMon);
	}
	
	public void removeMonomer (Monomer rmMon) {
		//rmMon.getGraphics().detach();
		rmMon.sepaku();
		rmMon = null;
	}
	
	public void step () {
		// simulate biochemistry over the next time-step
		
		end1BiochemSim ();			// catastrophy, polymerization, and depolymerization simulations for end1
		end2BiochemSim (); 			// and for end2
		
		sideBranchSim ();			// simulation of Arp2/3 mediated side-branching
		
		resetCounters();			// reset counters, zero force sums, etc in preparation for the next time step
	}
			
	
	public void end1BiochemSim () {
		if (!removeMe) { 
			if (end1Capped) {
				// check cap off
				if (!isSideBranch) {
					if (Rdm.uRdm() < Env.adpFactinArpOff.getValue()*deltaT) { capEnd1(false); }
				}
			} else {
				// Polymerization
				if (end1TipC > halfmono) {
					boolean monomerAdded = addMonomerSim(0);//Env.kMinusADP_On);
					if (monomerAdded) { 
						this.addMinusMonomer();
						coord.inc(-quartmono,uVec);
						end1.inc(-halfmono,uVec);
						adjustSideBranchLocs(halfmono);
					}
				}
				
				// Depolymerization
				boolean monomerRemoved = removeMonomerSim(Env.adpMinusOff.getValue());
				if (monomerRemoved) {
					this.removeMinusMonomer();
					coord.inc(quartmono,uVec);
					end1.inc(halfmono,uVec);
					adjustSideBranchLocs(-halfmono);
				}
				
				// Remove filament if less than trimer
				if (length < nuclength) { cleanupAndRemove(this); }
			}
		}
	}
	
	public void end2BiochemSim () {
		if (!removeMe){
			if (end2Capped) {
				// check end2 uncapping...
				if (Rdm.uRdm() < Env.capOff.getValue()*deltaT) { capEnd2(false); }
			} else {
				end2TipC = theCell.getTipClearance(end2,uVec);	// get clearance from piston surface
				
				// Polymerization
				if (end2TipC > Env.polyGap.getValue()) {
					boolean monomerAdded = addMonomerSim(Env.atpPlusOn.getValue());
					if (monomerAdded) { 
						this.addPlusMonomer();
						coord.inc(quartmono,uVec); 
						end2.inc(halfmono,uVec);
					}
				}
				
				// Depolymerization
				if (length > monosize) {
					boolean monomerRemoved = removeMonomerSim(Env.atpPlusOff.getValue());
					if (monomerRemoved) {
						this.removePlusMonomer();
						coord.inc(-quartmono,uVec);
						end2.inc(-halfmono,uVec);
					}
				}
				
				// Capping
				double capProbMod = 1;
				if (Env.surfaceBiasesCapping) {
					if (end2TipC < Env.capBiasDistance*Env.capDiameter) {
						capProbMod = 0;//(end2TipC/Env.radOfCap);
					}
				}
				if (Rdm.uRdm() < capProbMod*Env.capOn.getValue()*theCell.getCapConc(end2)*deltaT) {
					capEnd2(true);
				}
			}
		}
	}
	
	public void capEnd1 (boolean onORoff) {
		end1Capped = onORoff;
		if (end1Capped) { theCell.takeArp(1); } else { theCell.putArp(1); }
		updateGraphicsCapState = true;
	}
	
	public void capEnd2 (boolean onORoff) {
		end2Capped = onORoff;
		if (end2Capped) { theCell.takeCap(1); } else { theCell.putCap(1); }
		updateGraphicsCapState = true;
	}
	
	public void setSideBranch (boolean isORnot) {
		isSideBranch = isORnot;
	}
	
	public boolean tooCloseBranch (double branchLoc) {
		double minDist = Env.branchMinDist.getValue();
		if (length < minDist) { return true; }
		for (int i = 0; i < branchCt; i++) {
			if (Math.abs(branchLoc-branchLocs[i]) < minDist) { return true; }
		}
		return false;
	}
	
	public void addBranch (Filament sideBranch, double branchLoc) {
		// make a new Arp2/3 formed branch at the given location
		// check if we need to expand the Arp array
		if (branchCt >= branches.length) {
			Filament [] tempBranchArray = branches;
			double [] tempBranchLocArray = branchLocs;
			branches = new Filament [branchCt + 5];
			branchLocs = new double [branchCt + 5];
			for (int i=0;i<branchCt;i++) {
				branches[i] = tempBranchArray[i];
				branchLocs[i] = tempBranchLocArray[i];
			}
			tempBranchArray = null;
			tempBranchLocArray = null;
		}
		branches[branchCt] = sideBranch;
		branchLocs[branchCt] = branchLoc;
		branchCt++;
	}
	
	public void sideBranchSim () {
		end2TipC = theCell.getTipClearance(end2,uVec);	// get clearance from piston surface
		double branchProbMod = 1;
		if (Env.branchOnlyAtSurface) {
			if (end2TipC < Env.branchZoneFromSurface*Env.arpDiameter) { 
				branchProbMod = 1;		// allow branching if within specified branching zone
			} else {
				branchProbMod = 0;		// otherwise.. no branching
			}
		}
		double branchProb = branchProbMod*Env.arpSideBranching.getValue()*theCell.getArpAConc()*deltaT;
		if (Rdm.uRdm() < branchProb) {
			double branchLoc;
			if (length < Env.branchBarbedZone.getValue()) {
				branchLoc = Rdm.uRdm()*length; 
			} else {
				branchLoc = length + (Rdm.uRdm()-1)*Env.branchBarbedZone.getValue();  // measured from end1
			}
			if (! tooCloseBranch(branchLoc)) {
				Pt3D branchPt = Pt3D.Add(end1,branchLoc,uVec);
				Pt3D branchAng = makeBranchAng(uVec,t1Vec,t2Vec);
				Filament sideBranch = new Filament (branchPt,branchAng,monosize);
				sideBranch.slideFilament(end1BranchOffset);	// translate filament along chosen angle
				sideBranch.capEnd1(true);
				sideBranch.setSideBranch(true);
				addBranch(sideBranch,branchLoc);
			}
		}
	}
	
	public static Pt3D makeBranchAng (Pt3D uVec1, Pt3D t1Vec, Pt3D t2Vec) {
		// first find the magnitude of the branch angle for this particular branch
		double bAngVar = (2*Rdm.uRdm()-1)*(Env.branchAngVar.getValue()*Env.degToRad);
		double bAng = Env.branchAng.getValue()*Env.degToRad +bAngVar;
		double tAng = Rdm.uRdm()*2*Math.PI;
		Pt3D baseVec = Pt3D.Add(Math.cos(tAng), t1Vec, Math.sin(tAng), t2Vec);
		Pt3D uVec2 = Pt3D.Add(Math.cos(bAng), uVec1, Math.sin(bAng), baseVec);
		Pt3D branchAngles = Pt3D.EulerAnglesFromUVec(uVec2);
		return branchAngles;
	}
	
	public void adjustSideBranchLocs (double delta) {
		// this method should be called whenever the pointed-end polys or depolys
		for (int i=0;i<branchCt;i++) {
			branchLocs[i] += delta;
		}
		detachSideBranches();
	}
	
	public void detachSideBranches () {
		for (int i=0;i<branchCt;i++) {
			if (branches[i] == null) { break; } // leave loop if we prematurely reach end of list
			if ((branchLocs[i] <= 0) | (branchLocs[i] > length)) {
				branches[i].setSideBranch(false);		// remove cap from now unbound side-branch
				branches[i] = branches[branchCt-1];		//swap last array entry into now open slot
				branchLocs[i] = branchLocs[branchCt-1];
				branches[branchCt-1] = null;
				branchLocs[i] = 0;
				branchCt --;
			}
		}
	}
	
	public void releaseAllSideBranches () {
		for (int i=0;i<branchCt;i++) {
			branches[i].setSideBranch(false);
			branches[i] = null;
			branchLocs[i] = 0;
		}
		branchCt = 0;
	}
			
	
	public void resetCounters() {
		super.resetCounters();	// call the generic Thing method
		end1TipC = 1e6;			
		end2TipC = 1e6;
	}
	
	public boolean addMonomerSim (double onRate){
		double onProb = onRate*theCell.getActinConc()*deltaT;
		if ((Rdm.uRdm()< onProb) & (theCell.isActin(1))){
			length+=halfmono;
			theCell.takeActin(1);
			updateGraphicsLength = true;
			return true;
		}
		return false;
	}
	
	public boolean removeMonomerSim (double offRate) {
		if (Rdm.uRdm()< offRate*deltaT){
			length+= -halfmono;
			theCell.putActin(1);
			updateGraphicsLength = true;
			return true;
		}
		return false;
	}
	
	public static void cleanupAndRemove (Filament cleanF) {
		cleanup(cleanF);
		removeFilament(cleanF);
	}
		
	public static void cleanup (Filament cleanF) {
			int lInt = (int)(cleanF.length/halfmono);
			theCell.putActin(lInt);
			if (cleanF.end1Capped) { theCell.putArp(1); }
			if (cleanF.end2Capped) { theCell.putCap(1); }
			cleanF.releaseAllSideBranches();
	}
	
	public static void removeAll () {
		for (int i=0;i<filamentCt;i++) {
			cleanup(theFilaments[i]);
			theFilaments[i].G.detach();
		}
		for (int i=0;i<filamentCt;i++) {
			theFilaments[i].removeMe = true;
			theFilaments[i] = null;
		}
		filamentCt = 0;
	}
	
	public static Stat branchStatistics () {
		Filament curFil;
		double curDist;
		double branchDistSum = 0;
		double branchDistSqrdSum = 0;
		int branchNum = 0;
		for (int i=0;i<filamentCt;i++) {
			double curLoc = 0;
			curFil = theFilaments[i];
			for (int j=0;j<curFil.branchCt;j++) {
				curDist = curFil.branchLocs[j] - curLoc;
				branchDistSum += curDist;
				branchDistSqrdSum += curDist*curDist;
				branchNum++; 
				curLoc = curFil.branchLocs[j];
			}
		}
		double branchMean = branchDistSum/branchNum;
		double branchStdDev = Math.sqrt((branchDistSqrdSum - (branchDistSum*branchDistSum/branchNum))/(branchNum-1));
		Stat branchStat = new Stat (branchMean,branchStdDev);
		return branchStat;
	}
	
	public static Stat filLengthStatistics () {
		double curLength;
		double filLengthSum = 0;
		double filLengthSqrdSum = 0;
		int filNum = 0;
		for (int i=0;i<filamentCt;i++) {
			curLength = theFilaments[i].length;
			filLengthSum += curLength;
			filLengthSqrdSum += curLength*curLength;
			filNum++;
		}
		double lengthMean = filLengthSum/filNum;
		double lengthStdDev = Math.sqrt((filLengthSqrdSum - (filLengthSum*filLengthSum/filNum))/(filNum-1));
		Stat filStat = new Stat (lengthMean,lengthStdDev);
		return filStat;
	}
	
	public static Stat filAnglePlusStatistics () {
		double curAng;
		double filAngSum = 0;
		double filAngSqrdSum = 0;
		int filNum = 0;
		for (int i=0;i<filamentCt;i++) {
			if (theFilaments[i].uVec.x > 0) {
				curAng = Env.radToDeg*Math.acos(theFilaments[i].uVec.x);
				filAngSum += curAng;
				filAngSqrdSum += curAng*curAng;
				filNum++;
			}
		}
		double angMean = filAngSum/filNum;
		double angStdDev = Math.sqrt((filAngSqrdSum - (filAngSum*filAngSum/filNum))/(filNum-1));
		Stat filStat = new Stat (angMean,angStdDev);
		return filStat;
	}
	
	public static Stat filAngleMinusStatistics () {
		double curAng;
		double filAngSum = 0;
		double filAngSqrdSum = 0;
		int filNum = 0;
		for (int i=0;i<filamentCt;i++) {
			if (theFilaments[i].uVec.x < 0) {
				curAng = Env.radToDeg*Math.acos(theFilaments[i].uVec.x);
				filAngSum += curAng;
				filAngSqrdSum += curAng*curAng;
				filNum++;
			}
		}
		double angMean = filAngSum/filNum;
		double angStdDev = Math.sqrt((filAngSqrdSum - (filAngSum*filAngSum/filNum))/(filNum-1));
		Stat filStat = new Stat (angMean,angStdDev);
		return filStat;
	}
	
	
	public static void makeInitialFilaments () {
		int initNuc = (int)(Env.initSeeds.getValue());
		for (int i=0;i<initNuc;i++) { Filament.makeRandomBasalFilament();}
	}
	
	public static void makeRandomBasalFilament () {
		if (theCell.isActin(Filament.nucCount)) {
			Pt3D randomLoc = theCell.rdmBasalPt();
			Pt3D orient = new Pt3D(Rdm.uRdm()*Math.PI/2,2*Math.PI*Rdm.uRdm(),0);
			Filament newFil = new Filament (randomLoc, orient);
			newFil.slideFilament(nuclength/2);
			if (theCell.isArp(1)) { newFil.capEnd1(true); } else { talkln ("Cannot cap pointed-end of basal filament..  insufficient Arp2/3"); }
		} else {
			talkln ("Could not make basal filament.. insufficient actin");
		}
	}
	
	public static void makeRandomFilament () {
		Pt3D randomLoc = theCell.rdmPtInsideCell();
		Pt3D randomOrient = Pt3D.Scale(2*Math.PI,Pt3D.RandomPositive());
		new Filament (randomLoc,randomOrient);
	}
	public static void makeRandomFilament (double length) {
		int lInt = (int)(length/halfmono);
		if (theCell.isActin(lInt)) {
			Pt3D randomLoc = theCell.rdmPtInsideCell();
			Pt3D randomOrient = Pt3D.Scale(2*Math.PI,Pt3D.RandomPositive());
			new Filament (randomLoc,randomOrient,length);
		}
	}
	
	public static void addFilament (Filament newFilament) {
		theFilaments[filamentCt] = newFilament;
		newFilament.myFilamentNber = filamentCt;
		filamentCt ++;
	}
	public static void removeFilament ( Filament byeFilament) {
		int swapId = byeFilament.myFilamentNber;
		theFilaments[swapId] = theFilaments[filamentCt-1];
		theFilaments[swapId].myFilamentNber = swapId;
	//	byeFilament = null;
		filamentCt += -1;
		byeFilament.removeMe = true;
	
	}
	
	public void gotPlasmidAppearance(int endWithPlasmid) {
		if (!graphicsMade) { makeGraphics(); }
		ColoringAttributes cA = new ColoringAttributes(gotPlasmidColor, ColoringAttributes.NICEST);
		a.setColoringAttributes(cA);
		
		filShape.setAppearance(a);
	}
	
	private void makeNewCyl () {
		int cylXTess = Env.lowCylXTessalation;
		int cylYTess = Env.lowCylYTessalation;
		int transMode = TransparencyAttributes.NONE;
		switch (Env.graphicsQuality) {
		case Env.LOW_QUALITY:
			cylXTess = Env.lowCylXTessalation;
			cylYTess = Env.lowCylYTessalation;
			//transMode = TransparencyAttributes.NONE;
			break;
		case Env.MID_QUALITY:
			cylXTess = Env.midCylXTessalation;
			cylYTess = Env.midCylYTessalation;
			//transMode = TransparencyAttributes.FASTEST;
			break;
		case Env.HIGH_QUALITY:
			cylXTess = Env.highCylXTessalation;
			cylYTess = Env.highCylYTessalation;
			//transMode = TransparencyAttributes.NICEST;
			break;
		}
		//Appearance 
		Material cylM = new Material(ambientC,emissiveC,cylColor,specularC,shiny);
		aCyl = new Appearance();
		TransparencyAttributes tA = new TransparencyAttributes ();
		tA.setTransparency(0.6f);
		tA.setTransparencyMode(transMode);
		aCyl.setTransparencyAttributes(tA);
		aCyl.setMaterial(cylM);
		
		//cylindrical body
		cylBG = new BranchGroup();
		cylBG.setCapability(BranchGroup.ALLOW_DETACH);
		myCyl = new Cylinder((float)radius,(float)length,Primitive.GENERATE_NORMALS,cylXTess,cylYTess,aCyl);
		Transform3D cylRot = new Transform3D();
		cylRot.rotZ(Math.PI/2);
		cylT3D = new Transform3D();
		cylT3D.setRotation(mxToX);
		cylT3D.mul(cylRot);
		Pt3D cylCen = Pt3D.Add(end1,length/2,uVec);
		cylT3D.setTranslation(new Vector3d(cylCen.x,cylCen.y,cylCen.z));
		cylTG = new TransformGroup(cylT3D);
		cylTG.addChild(myCyl);
		cylBG.addChild(cylTG);
		G.addChild(cylBG);
	}
	
	public void makeInitMonomers () {
		// create the initial monomers
		int initMons = (int)((length-halfmono)/halfmono);
		Monomer curMon,lastMon = null;
		Pt3D curPt = new Pt3D(end1);
		for (int i=0;i<initMons;i++) {
			curPt.inc(halfmono,uVec);
			curMon = addMonomer(curPt);
			if (i==0) {
				curMon.minusMon = null;
				curMon.plusMon = null;
				minusMon = curMon;
				lastMon = curMon;
			}
			if (i==initMons-1) {
				if (i!=0) {
					curMon.minusMon = lastMon;
					lastMon.plusMon = curMon;
				}
				plusMon = curMon;
			}
			if ((i>0) & (i<initMons-2)) {
				curMon.minusMon = lastMon;
				lastMon.plusMon = curMon;
			}
		}
	}
	
	public static void eraseOldGraphics () {
		Monomer curMon;
		for (int i=0;i<filamentCt;i++) {
			theFilaments[i].G.detach();
			theFilaments[i].G.removeAllChildren();
			theFilaments[i].updateGraphicsLength = true;
			theFilaments[i].updateGraphicsCapState = true;
			
			curMon = theFilaments[i].minusMon;
			boolean noMoreMons = false;
			while (!noMoreMons) {
				curMon.eraseOldGraphics();
				if (curMon.plusMon != null) {
					curMon = curMon.plusMon;
				} else {
					noMoreMons = true;
				}
			}
			theFilaments[i].graphicsMade = false;
		}
	}

	public void makeGraphics () {
		// set graphics quality
		int sphereTessalation = Env.lowSphereTessalation;
		int transMode = TransparencyAttributes.NONE;
		switch (Env.graphicsQuality) {
		case Env.LOW_QUALITY:
			sphereTessalation = Env.lowSphereTessalation;
			//transMode = TransparencyAttributes.NONE;
			break;
		case Env.MID_QUALITY:
			sphereTessalation = Env.midSphereTessalation;
			//transMode = TransparencyAttributes.FASTEST;
			break;
		case Env.HIGH_QUALITY:
			sphereTessalation = Env.highSphereTessalation;
			//transMode = TransparencyAttributes.NICEST;
			break;
		}
	
		
		// make material
		m = new Material (ambientC,emissiveC,diffuseC,specularC,shiny);
		m.setCapability(Material.ALLOW_COMPONENT_WRITE);
		capM = new Material(ambientC,emissiveC,capBoxColor,specularC,shiny);
		capM.setCapability(Material.ALLOW_COMPONENT_WRITE);
		arpM = new Material(ambientC,emissiveC,arpSphColor,specularC,shiny);
		arpM.setCapability(Material.ALLOW_COMPONENT_WRITE);
		
		// set capabilities
		setGraphicsCapabilities();
		
		// appearance for lines connecting spheres
		ColoringAttributes cA = new ColoringAttributes(diffuseC, ColoringAttributes.NICEST);
		a.setLineAttributes(new LineAttributes(1.0f,LineAttributes.PATTERN_SOLID,true));
		a.setColoringAttributes(cA);
		//a.setPolygonAttributes(new PolygonAttributes(PolygonAttributes.POLYGON_LINE,PolygonAttributes.CULL_NONE,0.0f));
		
		// appearance for spheres
		aSph = new Appearance();
		aSph.setMaterial(m);
		
		// appearance for cap
		aCap = new Appearance();
		aCap.setMaterial(capM);
		
		// appearance for arp
		aArp = new Appearance();
		TransparencyAttributes tA = new TransparencyAttributes ();
		tA.setTransparency(0.6f);
		tA.setTransparencyMode(transMode);
		aArp.setTransparencyAttributes(tA);
		aArp.setMaterial(arpM);
		
		
		// cylindrical body
		cylBG = new BranchGroup();
		cylBG.setCapability(BranchGroup.ALLOW_DETACH);
		if (Env.filamentCyls) { makeNewCyl(); }
		
		// line array
		theFil = new LineArray(2,LineArray.COORDINATES);
		theFil.setCapability(theFil.ALLOW_COORDINATE_WRITE);
		theFil.setCoordinate(0,new Point3d(end1.x,end1.y,end1.z));
		theFil.setCoordinate(1,new Point3d(end2.x,end2.y,end2.z));
		filShape = new Shape3D();
		filShape.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		filShape.setCapability(filShape.ALLOW_GEOMETRY_WRITE);
		filShape.setGeometry(theFil);
		filShape.setAppearance(a);
		
		g3d = new TransformGroup();
		g3d.setTransform(t3d);
		g3d.addChild(filShape);
		
		// capping sphere
		float capDim = (float)(Env.radOfCap);
		capS = new Sphere (capDim,Sphere.GENERATE_NORMALS,sphereTessalation,aCap);
		capS.setCapability(capS.ENABLE_APPEARANCE_MODIFY);
		capT3D = new Transform3D();
		capT3D.setTranslation(farFarAway);	// off in space somewhere far far away ...hack
		capTG = new TransformGroup(capT3D);
		capTG.setCapability(capTG.ALLOW_TRANSFORM_WRITE);
		capTG.addChild(capS);
		
		// arpSphere
		float arpDim = (float)(Env.radOfARP);
		arpS = new Sphere (arpDim,Sphere.GENERATE_NORMALS, sphereTessalation, aArp);
		arpS.getShape().setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		arpT3D = new Transform3D();
		arpT3D.setTranslation(farFarAway);
		arpTG = new TransformGroup(arpT3D);
		arpTG.setCapability(arpTG.ALLOW_TRANSFORM_WRITE);
		arpTG.addChild(arpS);
		
		// end1Sphere
		float actDim = (float)(Env.radOfActin);
		end1S = new Sphere (actDim,Sphere.GENERATE_NORMALS, sphereTessalation, aSph);
		end1S.getShape().setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		end1T3D = new Transform3D();
		end1T3D.setTranslation(farFarAway);
		end1TG = new TransformGroup(end1T3D);
		end1TG.setCapability(end1TG.ALLOW_TRANSFORM_WRITE);
		end1TG.addChild(end1S);
		
		// end2Sphere
		end2S = new Sphere (actDim,Sphere.GENERATE_NORMALS, sphereTessalation, aSph);
		end2S.getShape().setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		end2T3D = new Transform3D();
		end2T3D.setTranslation(farFarAway);
		end2TG = new TransformGroup(end2T3D);
		end2TG.setCapability(end2TG.ALLOW_TRANSFORM_WRITE);
		end2TG.addChild(end2S);
		
		G.addChild(g3d);
		G.addChild(end1TG);
		G.addChild(end2TG);
		G.addChild(capTG);
		G.addChild(arpTG);
		graphicsMade = true;
	}
	public void updateGraphics () {
		if (updateGraphicsLength) {
			if (Env.filamentLines) {
				theFil.setCoordinate(0,new Point3d(end1.x,end1.y,end1.z));
				theFil.setCoordinate(1,new Point3d(end2.x,end2.y,end2.z));
				filShape.setGeometry(theFil);
			} else {
				theFil.setCoordinate(0,farfarAwayPt);
				theFil.setCoordinate(1,farfarAwayPt);
				filShape.setGeometry(theFil);
			}
		
			
			end1T3D.setTranslation(new Vector3d(end1.x,end1.y,end1.z));
			end1TG.setTransform(end1T3D);
			end2T3D.setTranslation(new Vector3d(end2.x,end2.y,end2.z));
			end2TG.setTransform(end2T3D);
			
			G.removeChild(cylBG);
			if (Env.filamentCyls) { makeNewCyl(); }
			
			updateGraphicsLength = false;
		}
		
		if (updateGraphicsCapState) {
			if (end2Capped) {
				capT3D.setTranslation(new Vector3d(end2.x,end2.y,end2.z));
				capTG.setTransform(capT3D);
			} else {
				capT3D.setTranslation(farFarAway);
				capTG.setTransform(capT3D);
			}
			
			if (end1Capped) {
				Pt3D end1CapPt = Pt3D.Add(end1,-end1CapOffset,uVec);
				arpT3D.setTranslation(new Vector3d(end1CapPt.x,end1CapPt.y,end1CapPt.z));
				arpTG.setTransform(arpT3D);
			} else {
				arpT3D.setTranslation(farFarAway);
				arpTG.setTransform(arpT3D);
			}
			
			updateGraphicsCapState = false;
		}
	 }
	
		
}


