package actinScope;
/* Parameter... a class for storing / changing parameter values */

/*  <ActinScope - for the simulation of dendritic actin networks>
    Copyright (C) <2005>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

public class Parameter {
	static final int DOUBLE = 0;
	static final int INT = 1;
	int type;
	String parameterName;
	String units;
	double defaultValue;
	double curValue;
	double lastValue;
	boolean positiveValue = true;		// by default.. only positive values for parameters
	boolean depletable = true;
	
	public Parameter (String parameterName, double defaultValue, String units) {
		this.parameterName = parameterName;
		this.defaultValue = defaultValue;
		curValue = defaultValue;
		setLastValue(curValue);
		this.units = units;
		this.type = DOUBLE;
	}
	
	public Parameter (String parameterName, double defaultValue, String units,int type) {
		this.parameterName = parameterName;
		this.defaultValue = defaultValue;
		curValue = defaultValue;
		setLastValue(curValue);
		this.units = units;
		this.type = type;
	}
	
	public Parameter (String parameterName, double defaultValue, String units,boolean depletable) {
		this.parameterName = parameterName;
		this.defaultValue = defaultValue;
		curValue = defaultValue;
		setLastValue(curValue);
		this.units = units;
		this.depletable = depletable;
	}
	
	public void setValue (double newValue) {
		curValue = newValue;
	}
	
	public double getValue () {
		return curValue;
	}
	
	
	public void addToValue (double addVal) {
		if (depletable) {
			curValue += addVal;
			if ((positiveValue) & (curValue < 0)) {
				curValue = 0;
			}
		}
	}
	
	public void resetToDefault () {
		curValue = defaultValue;
	}
	
	public void resetToLastValue () {
		curValue = lastValue;
	}
	
	public void setLastValue (double newLastVal) {
		lastValue = newLastVal;
	}
	
	public double getLastValue () {
		return lastValue;
	}
	
	public void setName (String name) {
		parameterName = name;
	}
	
	public String getName () {
		return parameterName;
	}
	
	public String getUnits () {
		return units;
	}
	
	public void setPositiveValueOnly (boolean pos) {
		positiveValue = pos;
	}
	
	public void setDepletable (boolean deplete) {
		depletable = deplete;
	}
}
